/*
 * C
 *
 * Copyright 2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "LLHAL_GPIO_impl.h"
#include "hal_k65.h"
#include "adc_manager.h"
#include "dac_manager.h"

/* Macros --------------------------------------------------------------------*/


/* Fields --------------------------------------------------------------------*/

static uint8_t initialized = 0;

/* Private functions ---------------------------------------------------------*/

/* API -----------------------------------------------------------------------*/

int32_t LLHAL_GPIO_IMPL_setPinMode(int32_t port, int32_t pin, int32_t mode) {
	if (!initialized) {
		hal_k65_initialize();

		if (dac_manager_initialize() != HAL_GPIO_OK) {
			// unknown error
			return HAL_GPIO_NOK;
		}

		if (adc_manager_initialize() != HAL_GPIO_OK) {
			// unknown error
			return HAL_GPIO_NOK;
		}

		initialized = !initialized;
	}

	hal_k65_gpio_data gpio;
	gpio_pin_config_t gpio_config = { 0 };
	port_pin_config_t port_config = { 0 };

	if (HAL_GPIO_OK != hal_k65_get_gpio_data(port, pin, &gpio)) {
		// Invalid setup
		return HAL_GPIO_NOK;
	}

	switch (mode) {

	case LLHAL_GPIO_MODE_DIGITAL_INPUT:
		port_config.mux = kPORT_MuxAsGpio;
		gpio_config.pinDirection = kGPIO_DigitalInput;
		break;

	case LLHAL_GPIO_MODE_DIGITAL_INPUT_PULLUP:
		port_config.mux = kPORT_MuxAsGpio;
		port_config.pullSelect = kPORT_PullUp;
		gpio_config.pinDirection = kGPIO_DigitalInput;
		break;

	case LLHAL_GPIO_MODE_DIGITAL_OUTPUT:
		port_config.mux = kPORT_MuxAsGpio;
		gpio_config.pinDirection = kGPIO_DigitalOutput;
		break;

	case LLHAL_GPIO_MODE_ANALOG_INPUT:

		if (gpio.adc_rank == HAL_GPIO_NOK) {
			// this is not a ADC pin
			return HAL_GPIO_NOK;
		}
		port_config.mux = kPORT_PinDisabledOrAnalog;
		port_config.pullSelect = kPORT_PullDisable;
		break;

	case LLHAL_GPIO_MODE_ANALOG_OUTPUT: {
		if (gpio.dac_data.DACx == NULL) {
			// this is not a DAC pin
			return HAL_GPIO_NOK;
		}
		break;
	}
	default: {
		return HAL_GPIO_NOK;
	}
	}

	if ((gpio.PORTx != NULL) && (gpio.GPIOx != NULL)) {
		PORT_SetPinConfig(gpio.PORTx, gpio.GPIO_Pin, &port_config);
		GPIO_PinInit(gpio.GPIOx, gpio.GPIO_Pin, &gpio_config);
	}

	return HAL_GPIO_OK;
}

int32_t LLHAL_GPIO_IMPL_getDigitalValue(int32_t port, int32_t pin)
{
	hal_k65_gpio_data gpio;

	if (HAL_GPIO_OK != hal_k65_get_gpio_data(port, pin, &gpio))
	{
		// Invalid setup
		return HAL_GPIO_NOK;
	}

	return GPIO_ReadPinInput(gpio.GPIOx, pin) == 0 ? LLHAL_GPIO_DIGITAL_LOW : LLHAL_GPIO_DIGITAL_HIGH;

}

int32_t LLHAL_GPIO_IMPL_setDigitalValue(int32_t port, int32_t pin, int32_t value)
{
	hal_k65_gpio_data gpio;

	if (HAL_GPIO_OK != hal_k65_get_gpio_data(port, pin, &gpio))
	{
		// Invalid setup
		return HAL_GPIO_NOK;
	}

	GPIO_WritePinOutput(gpio.GPIOx, gpio.GPIO_Pin, value == LLHAL_GPIO_DIGITAL_LOW ? 0 : 1);
	return HAL_GPIO_OK;
}

int32_t LLHAL_GPIO_IMPL_getAnalogValue(int32_t port, int32_t pin)
{
	hal_k65_gpio_data gpio;

	if (HAL_GPIO_OK != hal_k65_get_gpio_data(port, pin, &gpio)) {
		// Invalid setup
		return HAL_GPIO_NOK;
	}

	return adc_manager_get_value(&gpio);
}

int32_t LLHAL_GPIO_IMPL_setAnalogValue(int32_t port, int32_t pin, int32_t percentage)
{
	hal_k65_gpio_data gpio;

	if (HAL_GPIO_OK != hal_k65_get_gpio_data(port, pin, &gpio)) {
		// Invalid setup
		return HAL_GPIO_NOK;
	}

	return dac_manager_set_value(&gpio, percentage);
}

/* EOF -----------------------------------------------------------------------*/

