/*
 * C
 *
 * Copyright 2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "lcd_task.h"
#include "interrupts.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "task.h"
#include "lcd_manager.h"

/* Defines -------------------------------------------------------------------*/
#define LCD_TASK_PRIORITY ( 12 )
#define LCD_TASK_STACK_SIZE (128) // portSTACK_TYPE is 32 bits wide, so 128 * 4 = 512 bits

/* Global --------------------------------------------------------------------*/
xSemaphoreHandle lcd_task_copy_sem = NULL;
xSemaphoreHandle lcd_task_synchro_sem = NULL;

/* Private API ---------------------------------------------------------------*/
void LCD_TASK_copyBuffer(void)
{
  lcdc_fill_window_pic(copyParams.xmin, copyParams.ymin, copyParams.xmax, copyParams.ymax, copyParams.srcAddr);
  xSemaphoreGive(lcd_task_synchro_sem);
}

static void vLcdTaskFunction(void *p_arg)
 {
	LCD_MANAGER_LayerDefaultInit();
	while (1) {
		// Suspend ourselves
		xSemaphoreTake(lcd_task_copy_sem, portMAX_DELAY);

		LCD_TASK_copyBuffer();
	}
}

/* Interrupt function --------------------------------------------------------*/


/* API -----------------------------------------------------------------------*/

uint8_t LCD_TASK_create_task(void)
 {
	if ((lcd_task_copy_sem == NULL) || (lcd_task_synchro_sem == NULL)) {
		// create semaphores
		vSemaphoreCreateBinary(lcd_task_copy_sem);
		vSemaphoreCreateBinary(lcd_task_synchro_sem);
		xSemaphoreTake(lcd_task_copy_sem, 0);
		xSemaphoreTake(lcd_task_synchro_sem, 0);

		// create task
		xTaskCreate(vLcdTaskFunction, NULL, LCD_TASK_STACK_SIZE, NULL,
				LCD_TASK_PRIORITY, NULL);
	}

	return MICROEJ_TRUE;
}

