/*
 * C
 *
 * Copyright 2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#include "lcdc.h"

static void lcdc_send_data(uint16_t value) {
	*((volatile uint16_t*) LCDC_ADRESS_DATA) = value;
}

static void lcdc_send_cmd(uint16_t cmd) {
	*((volatile uint16_t*) LCDC_ADRESS_CMD) = cmd;
}

const uint16_t init_data[] =
{
    0x0000,  0x0001,

    0x0003,  0xAEAC, 0x000C,  0x0007, 0x000D,  0x000F, 0x000E,  0x2900, 0x001E,  0x00B3,

    0x0001,  0x2B3F, 0x0002,  0x0600, 0x0010,  0x0000, 0x0011,  0x60B0,

    0x0005,  0x0000, 0x0006,  0x0000, 0x0016,  0xEF1C, 0x0017,  0x0003, 0x0007,  0x0233,
    0x000B,  0x5308, 0x000F,  0x0000,

    0x0041,  0x0000, 0x0042,  0x0000, 0x0048,  0x0000, 0x0049,  0x013F, 0x0044,  0xEF00,
    0x0045,  0x0000, 0x0046,  0x013F, 0x004A,  0x0000, 0x004B,  0x0000,

    0x0030,  0x0707, 0x0031,  0x0704, 0x0032,  0x0204, 0x0033,  0x0201, 0x0034,  0x0203,
    0x0035,  0x0204, 0x0036,  0x0204, 0x0037,  0x0502, 0x003A,  0x0302, 0x003B,  0x0500,
    0x0023 , 0x0000, 0x0024 , 0x0000
};

static void lcdc_fill_initial_data(const uint16_t data[], uint32_t count) {
	uint32_t i;

	for (i = 0; i < count; i += 2) {
		lcdc_send_cmd(data[i]);
		lcdc_send_data(data[i + 1]);
	}
}

void lcdc_set_window(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2) {
	uint16_t x1_x2;
	uint16_t Addr1, Addr2;

	// landscape
	Addr1 = (uint16_t) (LCDC_SCREEN_PIXEL_Y - 1 - y1);
	Addr2 = x1;
	x1_x2 = (uint16_t) ((Addr1 << 8) + (LCDC_SCREEN_PIXEL_Y - 1 - y2));
	y1 = x1;
	y2 = x2;

	// set Window
	lcdc_send_cmd(0x0044);
	lcdc_send_data(x1_x2);

	lcdc_send_cmd(0x0045);
	lcdc_send_data(y1);

	lcdc_send_cmd(0x0046);
	lcdc_send_data(y2);

	// set Start Address counter
	lcdc_send_cmd(0x004e);
	lcdc_send_data(Addr1);

	lcdc_send_cmd(0x004f);
	lcdc_send_data(Addr2);

	lcdc_send_cmd(0x0022);
}

void lcdc_set_orientation(void) {
	lcdc_send_cmd(0x0011);
	lcdc_send_data(0x60a8);
}

void lcdc_fill_window(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2,
		uint16_t color) {
	uint16_t i, j;
	uint16_t height = y2 - y1;
	uint16_t width = x2 - x1;

	lcdc_set_window(x1, y1, x2, y2);

	for (i = 0; i <= height; i++) {
		for (j = 0; j <= width; j++) {
			lcdc_send_data(color);
		}
	}
}

void lcdc_init(void) {
	// Turn on oscillator
	lcdc_fill_initial_data(&init_data[0], 2);
	// Power control setup
	lcdc_fill_initial_data(&init_data[2], 10);
	//
	lcdc_fill_initial_data(&init_data[12], 8);
	lcdc_fill_initial_data(&init_data[20], 14);
	lcdc_fill_initial_data(&init_data[34], 18);
	lcdc_fill_initial_data(&init_data[52], 24);

	lcdc_set_orientation();
}

void lcdc_fill_window_pic(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2,
		uint16_t * data) {
	uint16_t i, j;
	uint16_t height = y2 - y1;
	uint16_t width = x2 - x1;
	uint32_t startOffset = (y1 * LCDC_SCREEN_PIXEL_X) + x1;

	lcdc_set_window(x1, y1, x2, y2);

	for (i = 0; i <= height; i++) {
		for (j = 0; j <= width; j++) {
			lcdc_send_data(data[startOffset + (i * LCDC_SCREEN_PIXEL_X) + j]);
		}
	}
}
