/*
 * C
 *
 * Copyright 2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#include "time_hardware_timer.h"
#include "pit.h"


/* Private variables ---------------------------------------------------------*/

static uint64_t software_counter;
static uint8_t isInitialized = 0;


/* Public functions ----------------------------------------------------------*/

void PIT0_IRQHandler(void)
{
    ++software_counter;

    PIT_ChnlFlagRst(kPIT_Chnl_0);
}

void time_hardware_timer_initialize(void)
{
    uint32_t clockSrc;
	// reset software counter
	software_counter = 0;

	PIT_Enable();
	PIT_FreezeEnable();

	// set a trigger event every microsecond
    clockSrc = CLOCK_GetFreq(kCLOCK_BusClk);
    PIT_ChnlSetVal(kPIT_Chnl_0, USEC_TO_COUNT(1u, clockSrc) - 1);

	// set the isr handler
    EnableIRQ(PIT0_IRQn);
        

	// enable interrupt
	PIT_ChnlIntEnable(kPIT_Chnl_0);
	PIT_ChnlFlagRst(kPIT_Chnl_0);

	// start timer
	PIT_ChnlTimerEnable(kPIT_Chnl_0);
        
        isInitialized = 1;
}

uint8_t time_hardware_timer_isInitialized(void)
{
  return isInitialized;
}

uint64_t time_hardware_timer_getCurrentTime(void)
{
	return software_counter/1000;
}

uint64_t time_hardware_timer_getTimeNanos(void)
{
	return software_counter*1000;
}

